"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
// Create a mock implementation of tcache
class MockTcache {
    constructor() {
        this.val = new Map();
        this.time = new Map();
        this.valPath = path_1.default.join('__dirname', 'val.json');
        this.timePath = path_1.default.join('__dirname', 'time.json');
        this.readMaps();
    }
    readMaps() {
        // Skip if files don't exist
        if (!fs_1.default.existsSync(this.valPath) || !fs_1.default.existsSync(this.timePath)) {
            return;
        }
        const valStr = fs_1.default.readFileSync(this.valPath, 'utf8');
        const timeStr = fs_1.default.readFileSync(this.timePath, 'utf8');
        const valArr = JSON.parse(valStr);
        const timeArr = JSON.parse(timeStr);
        valArr.forEach((item) => this.val.set(item[0], item[1]));
        timeArr.forEach((item) => this.time.set(item[0], item[1]));
    }
    writeMaps() {
        const valStr = JSON.stringify([...this.val]);
        const timeStr = JSON.stringify([...this.time]);
        fs_1.default.writeFileSync(this.valPath, valStr);
        fs_1.default.writeFileSync(this.timePath, timeStr);
    }
    set(key, value, ttl) {
        this.val.set(key, value);
        this.time.set(key, new Date(Date.now() + ttl));
    }
    get(key) {
        const now = new Date();
        const ttl = this.time.get(key);
        if (!ttl) {
            return undefined;
        }
        const ttlDate = new Date(ttl);
        if (ttlDate > now) {
            return this.val.get(key);
        }
        return undefined;
    }
}
// Mock the actual tcache module
jest.mock('../../../src/utils/tcache', () => MockTcache);
// Import the mocked module
const tcache_1 = __importDefault(require("../../../src/utils/tcache"));
// Mock fs module
jest.mock('fs', () => ({
    existsSync: jest.fn(),
    readFileSync: jest.fn(),
    writeFileSync: jest.fn(),
}));
// Mock path module
jest.mock('path', () => ({
    join: jest.fn().mockImplementation((...args) => args.join('/')),
}));
// Mock File enum
jest.mock('../../../src/utils', () => ({
    File: {
        VAL: 'val.json',
        TIME: 'time.json',
    },
}));
describe('tcache', () => {
    let cache;
    beforeEach(() => {
        jest.clearAllMocks();
        fs_1.default.existsSync.mockReturnValue(false);
        cache = new tcache_1.default();
    });
    describe('constructor', () => {
        it('should initialize with empty maps if files do not exist', () => {
            expect(fs_1.default.existsSync).toHaveBeenCalled();
            expect(fs_1.default.readFileSync).not.toHaveBeenCalled();
            expect(cache.val.size).toBe(0);
            expect(cache.time.size).toBe(0);
        });
        it('should load data from files if they exist', () => {
            // Mock file existence
            ;
            fs_1.default.existsSync.mockReturnValue(true);
            // Mock file content
            const mockValData = JSON.stringify([
                ['key1', 'value1'],
                ['key2', 'value2'],
            ]);
            const now = new Date();
            const future = new Date(now.getTime() + 10000);
            const mockTimeData = JSON.stringify([
                ['key1', now.toISOString()],
                ['key2', future.toISOString()],
            ]);
            fs_1.default.readFileSync.mockReturnValueOnce(mockValData).mockReturnValueOnce(mockTimeData);
            cache = new tcache_1.default();
            expect(fs_1.default.existsSync).toHaveBeenCalled();
            expect(fs_1.default.readFileSync).toHaveBeenCalledTimes(2);
        });
    });
    describe('set and get methods', () => {
        it('should set a value with TTL and retrieve it before expiration', () => {
            const key = 'testKey';
            const value = 'testValue';
            const ttl = 10000; // 10 seconds
            cache.set(key, value, ttl);
            expect(cache.val.get(key)).toBe(value);
            expect(cache.time.has(key)).toBe(true);
            const result = cache.get(key);
            expect(result).toBe(value);
        });
        it('should return undefined for expired values', () => {
            const key = 'expiredKey';
            const value = 'expiredValue';
            const ttl = -1000; // Already expired
            cache.set(key, value, ttl);
            const result = cache.get(key);
            expect(result).toBeUndefined();
        });
        it('should return undefined for non-existent keys', () => {
            const result = cache.get('nonExistentKey');
            expect(result).toBeUndefined();
        });
    });
    describe('writeMaps', () => {
        it('should write maps to files', () => {
            cache.set('key1', 'value1', 10000);
            cache.set('key2', 'value2', 20000);
            cache.writeMaps();
            expect(fs_1.default.writeFileSync).toHaveBeenCalledTimes(2);
            expect(fs_1.default.writeFileSync).toHaveBeenCalledWith(expect.any(String), expect.stringContaining('key1'));
            expect(fs_1.default.writeFileSync).toHaveBeenCalledWith(expect.any(String), expect.stringContaining('key2'));
        });
    });
});
//# sourceMappingURL=tcache.test.js.map